/* $Id */
/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2
  (included in the RadioActive distribution in doc/GPL) as published by
  the Free Software Foundation.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "guppi-bonobo-object.h"
#include "guppi-bonobo-view.h"

static void
guppi_object_update_view(BonoboView *view, gpointer user_data)
{
  GuppiBonoboView *view_data;

  view_data = (GuppiBonoboView*)gtk_object_get_data( GTK_OBJECT (view), "view_data");;
  guppi_bonobo_view_refresh(view_data);
}

static void
guppi_object_refresh_views(GuppiObject *obj)
{
  bonobo_embeddable_foreach_view(obj->bonobo_embeddable,
				 guppi_object_update_view, obj);
}

void guppi_object_init(GuppiObject *obj, BonoboEmbeddable *bonobo_embeddable)
{
  obj->bonobo_embeddable = bonobo_embeddable;
  obj->state = guppi_plot_state_new();

  /* let's have something to see */
  guppi_plot_state_add_element(obj->state, guppi_plot_element_newv("scatter", 0, NULL));
  guppi_plot_state_add_element(obj->state, guppi_plot_element_newv("graphpaper", 0, NULL));

}

void guppi_object_destroy(GuppiObject *obj)
{
  g_return_if_fail(obj != NULL);

  gtk_object_unref(GTK_OBJECT(obj->state));
  g_free(obj);
}

GuppiObject *
guppi_object_construct(BonoboEmbeddableFactory *this, void *data)
{
  BonoboEmbeddable *bonobo_embeddable;
  GuppiObject *obj;
  
  g_return_val_if_fail(obj = g_new0(GuppiObject, 1), NULL);
  
  /* Creates the BonoboObject server */
  bonobo_embeddable = bonobo_embeddable_new(guppi_bonobo_view_factory, obj);
  if (bonobo_embeddable == NULL) {
    g_free(obj);
    return NULL;
  }
  
  guppi_object_init(obj, bonobo_embeddable);
  
  return obj;
}
