/*
 * guppi-bonobo.c
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <gnome.h>
#include <glade/glade.h>
#include <libgnorba/gnorba.h>
#include <bonobo.h>
#include <guppi-useful.h>
#include <guppi-data-init.h>
#include <guppi-stat-init.h>
#include <guppi-plot-init.h>
#include <guppi-plot-plug-in.h>

#include "guppi-bonobo-object.h"
#include "guppi-bonobo-view.h"

#define PACKAGE "guppi-bonobo"
#define VERSION "0.0.0"

static BonoboObject *guppi_bonobo_factory(BonoboEmbeddableFactory*, void*);

static BonoboEmbeddableFactory *factory;
static int guppi_object_num = 0;

static void
guppi_bonobo_factory_init(int* argc, char **argv)
{
  CORBA_Environment ev;    
  CORBA_exception_init (&ev);

  gnome_CORBA_init_with_popt_table 
    (PACKAGE, VERSION,             /* Name of the component */
     argc, argv,                   /* Command-line arguments */
     NULL, 0, NULL,                /* popt table to parse arguments */
     GNORBA_INIT_SERVER_FUNC, &ev  /* GNORBA options */
     );
  
  if (!bonobo_init (CORBA_OBJECT_NIL,
		    CORBA_OBJECT_NIL,
		    CORBA_OBJECT_NIL))
    g_error (_("I could not initialize Bonobo"));
  
  CORBA_exception_free (&ev);
  
  factory = bonobo_embeddable_factory_new("bonobo-object-factory:guppi",
					  guppi_bonobo_factory, NULL);
}

static void
guppi_bonobo_destroy(BonoboEmbeddable *embeddable, GuppiObject *obj)
{
  /* Destroy the instance */
  guppi_object_destroy(obj);
  
  /* Destroy the factory if there are no more instances */
  if ((--guppi_object_num) <= 0)
    if (factory) {
      bonobo_object_unref(BONOBO_OBJECT (factory));
      factory = NULL;
      gtk_main_quit ();
    }   
}

static BonoboObject *
guppi_bonobo_factory(BonoboEmbeddableFactory *this, void *data)
{
  GuppiObject *obj;

  obj = guppi_object_construct(this, data);

  g_return_val_if_fail(obj != NULL, NULL);
  g_return_val_if_fail(obj->bonobo_embeddable != NULL, NULL);
  
  guppi_object_num++;
  
  /* Install destructor */
  gtk_signal_connect(GTK_OBJECT(obj->bonobo_embeddable), "destroy",
		     GTK_SIGNAL_FUNC(guppi_bonobo_destroy),
		     obj);
  
  return BONOBO_OBJECT(obj->bonobo_embeddable);
}

/****************************************************************************/

static void
load_rc_files(void)
{
  gchar buffer[1024];
  const gint buffer_len = 1024;

  g_snprintf(buffer, buffer_len, "%s/.guppibonoborc", gnome_util_user_home());
  if (g_file_exists(buffer)) {
    guppi_splash_message(_("Loading .guppibonoborc"));
    guppi_safe_load(buffer);
  }

  strncpy(buffer, "./.guppibonobolocalrc", buffer_len);
  if (g_file_exists(buffer)) {
    guppi_splash_message(_("Loading .guppibonobolocalrc"));
    guppi_safe_load(buffer);
  }
}

static void
real_main(void* closure, int argc, char* argv[])
{

  guppi_make_warnings_and_criticals_stop_in_debugger("libguppiscatter", NULL);

  guppi_splash_message(_("Initializing libglade"));
  glade_gnome_init();
  
  guppi_splash_message(_("Init bonobo"));  
  guppi_bonobo_factory_init(&argc, argv);

  guppi_splash_message(_("Initializing Utility Library"));
  guppi_useful_init();

  guppi_splash_message(_("Initializing Statistics Library"));
  guppi_stat_init();

  guppi_splash_message(_("Initializing Data Library"));
  guppi_data_init();

  guppi_splash_message(_("Initializing Plot Library"));
  guppi_plot_init();

  guppi_splash_message(_("Loading Plug-Ins"));
  guppi_plug_in_load_all();

  load_rc_files();

  guppi_splash_destroy();

  bonobo_main();
}

int
main(int argc, char* argv[])
{
  /*
    We need to start up enough of the toolkit to let us put up our
     spash screen.  We'll init the rest of gnome in just a little while.
  */
  gtk_init(&argc, &argv);
  gdk_imlib_init();

#if 0
  bindtextdomain(PACKAGE, GNOMELOCALEDIR);
  textdomain(PACKAGE);
#endif

  guppi_splash_create();

  guppi_splash_message(_("Booting Guile"));
  scm_boot_guile(argc, argv, real_main, NULL);
  g_assert_not_reached();
  return 1;
}


/* $Id: guppi.c,v 1.22 2000/04/21 23:22:42 trow Exp $ */
